@@ -172,7 +172,7 @@ struct psp {
     double
     get_g( pos_t seq, pos_t seq_pos ) = 0;
 
-    /// get the bound on the term
+    /// get the upper bound on the term: log(1-g)
     virtual
     double
     get_term_bound( top_down_it it ) = 0;
@@ -281,7 +281,7 @@ protected:
         // store bound on term if it is not 0.
         if( min_g > 0. ) {
             // term will be infinite if min_g is 1.
-            static_cast< Derived * >( this )->store_term_bound( v, std::log( 1. + min_g / ( 1. - min_g ) ) );
+            static_cast< Derived * >( this )->store_term_bound( v, std::log( 1. - min_g ) );
         }
 
         // return the minimum g for this sub-tree
