@@ -67,6 +67,17 @@ struct null_model {
 		OutputIt output ) const
 	{
 	}
+
+    template< typename InputIt, typename OutputIt >
+    InputIt
+    evaluate_in_context(
+        InputIt context,
+        InputIt begin,
+        InputIt end,
+        OutputIt output
+    ) const {
+        return begin;
+    }
 };
 
 /// Select lower order Markov model.
@@ -425,22 +436,50 @@ struct complete_markov_model {
 	markov_model< Order, AlphabetSize, Float, Scale > mm;
 	typename detail::lower_order_selector< Order, AlphabetSize, Float, Scale >::type lower_orders;
 
-	/// Evaluate the sequence given by [begin, end) and output cumulative probabilities into the output iterator
-	template< typename InputIt, typename OutputIt >
-	void
-	evaluate( InputIt begin, InputIt end, OutputIt output ) const {
-		const typename std::iterator_traits< InputIt >::difference_type length = end - begin;
-		const size_t order = traits::order();
-		_consume_lower_orders( begin, length, output );
-		if( order < size_t( length ) ) {
-			end -= order;
-			for( ++begin; begin != end; ++begin ) {
-				const double likelihood = mm.evaluate( begin );
-				*output = likelihood;
-				++output;
-			}
-		}
-	}
+    /// Evaluate the sequence given by [begin, end) and output cumulative probabilities into the output iterator
+    template< typename InputIt, typename OutputIt >
+    void
+    evaluate( InputIt begin, InputIt end, OutputIt output ) const {
+        const typename std::iterator_traits< InputIt >::difference_type length = end - begin;
+        const size_t order = traits::order();
+        _consume_lower_orders( begin, length, output );
+        if( order < size_t( length ) ) {
+            end -= order;
+            for( ++begin; begin != end; ++begin ) {
+                const double likelihood = mm.evaluate( begin );
+                *output = likelihood;
+                ++output;
+            }
+        }
+    }
+
+    /// Evaluate the sequence given by [begin, end) but preceded by data starting at context.
+    /// Output cumulative probabilities into the output iterator
+    template< typename InputIt, typename OutputIt >
+    InputIt
+    evaluate_in_context(
+        InputIt context,
+        InputIt begin,
+        InputIt end,
+        OutputIt output
+    ) const {
+        // if we cannot evaluate the beginning at this order, go to a lower order
+        if( context + traits::order() > begin) {
+            begin = lower_orders.evaluate_in_context( context, begin, std::min( end, context + traits::order() ), output );
+        }
+
+        // evaluate what we can at this order
+        begin -= traits::order();
+        end -= traits::order();
+        while( begin < end ) {
+            const double likelihood = mm.evaluate( begin );
+            *output = likelihood;
+            ++output;
+            ++begin;
+        }
+
+        return begin + traits::order();
+    }
 
 	template< size_t LowerOrder >
 	complete_markov_model< LowerOrder, AlphabetSize, Float, Scale > &
@@ -448,21 +487,20 @@ struct complete_markov_model {
 		return _get_lower_order_model< LowerOrder, Order, AlphabetSize, Float, Scale >()( *this );
 	}
 
-	template< typename InputIt, typename OutputIt >
-	void
-	_consume_lower_orders(
-		InputIt begin,
-		typename std::iterator_traits< InputIt >::difference_type length,
-		OutputIt output ) const
-	{
-		lower_orders._consume_lower_orders( begin, length, output );
-		if( typename std::iterator_traits< InputIt >::difference_type( traits::order() ) < length ) {
-			const double likelihood = mm.evaluate( begin );
-			*output = likelihood;
-			++output;
-		}
-	}
-
+    template< typename InputIt, typename OutputIt >
+    void
+    _consume_lower_orders(
+        InputIt begin,
+        typename std::iterator_traits< InputIt >::difference_type length,
+        OutputIt output ) const
+    {
+        lower_orders._consume_lower_orders( begin, length, output );
+        if( typename std::iterator_traits< InputIt >::difference_type( traits::order() ) < length ) {
+            const double likelihood = mm.evaluate( begin );
+            *output = likelihood;
+            ++output;
+        }
+    }
 };
 
 /// Get the lower order model.
