@@ -1755,7 +1755,7 @@ class SequenceSet(object):
     def __init__(self, fasta_filename, options):
         self.options = options
         self.num_bases, self.seqs, self.ids, self.index = read_sequences(fasta_filename, self.options)
-        self.index = build_index(self.seqs)
+        # self.index = build_index(self.seqs) # already done in read_sequences()
         self.occs = occurrences_from_index(self.index)
         self.freqs = ZeroOrderFrequencies(list(self.occs[:4]))
         self.freqs_with_pseudo_counts = self.freqs.add_pseudo_counts(self.options.back_dist_prior)
@@ -1791,7 +1791,7 @@ class SequenceSet(object):
         Return a background model of the sequences for the specified width.
         """
         if W not in self.bg_models:
-            self.bg_models[W] = BackgroundModelFromLikelihoods(self.data, self.lls, self.freqs_with_pseudo_counts, W)
+            self.bg_models[W] = create_bg_model_from_base_likelihoods(W, self.data, self.lls, self.freqs_with_pseudo_counts)
         return self.bg_models[W]
 
 
